<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2009, Pawel Preneta <jusuff@jabster.pl>
 * @link http://code.zikula.org/bianor/
 * @version $Id: content.php 139 2009-10-04 20:47:01Z jusuff $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package menutree
 */

/**
 * Return content pages
 *
 * Syntax used in menutree
 * {menutree:ext:content[:groupby=page&parent=1]}
 * Params in [] are optional and
 *      groupby = menuitem (default) or page, all other values stands for none
 *      parent - id of parent node - this allows to get specified node of content pages
 *
 *
 * @param array $args['item'] menu node to be replaced
 * @param string $args['lang'] current menu language
 * @param string $args['extrainfo'] additional params
 * @return mixed array of links if successful, false otherwise
 */
function menutree_externalapi_content($args)
{
    $item       = isset($args['item']) && !empty($args['item']) ? $args['item'] : null;
    $lang       = isset($args['lang']) && !empty($args['lang']) ? $args['lang'] : null;
    $extrainfo  = isset($args['extrainfo']) && !empty($args['extrainfo']) ? $args['extrainfo'] : null;
    // $item ang lang params are required
    if(!$item || !$lang) {
        return false;
    }

    // is there is extrainfo - convert it into array, parse_str is quite handy
    if($extrainfo) {
        parse_str($extrainfo, $extrainfo);
    }
    $extrainfo['parent'] = isset($extrainfo['parent']) ? (int)$extrainfo['parent'] : 0;
    $extrainfo['groupby'] = isset($extrainfo['groupby'])? $extrainfo['groupby'] : 'menuitem';

    // get id for first element, use api func to aviod id conflicts inside menu
    $idoffset = pnModAPIFunc('menutree','user','getidoffset',array('item' => $item['id']));
    $lineno = 0;

    $links = array();

    // if $extrainfo['group'] if false - don't group pages
    if($extrainfo['groupby'] == 'menuitem') {
        $links['content'] = array(
            $lang => array(
                'id' => $idoffset++,
                'name' => $item['name'],
                'href' => pnModUrl('Content'),
                'title' => $item['title'],
                'className' => $item['className'],
                'state' => $item['state'],
                'lang' => $lang,
                'lineno' => $lineno++,
                'parent' => $item['parent']
            )
        );
    }
    // need to set parent node id according to groupby mode
    $parentNode = $extrainfo['groupby'] == 'menuitem' ? $links['content'][$lang]['id'] : $item['parent'];

    // set option and get page list
    $options = array(
        'orderBy' => 'setLeft',
        'makeTree' => false,
        'language' => $lang,
        'filter' => array(
            'superParentId' => $extrainfo['parent']
        )
    );
    $pages = pnModAPIFunc('content', 'page', 'getPages', $options);

    $blocked = array();
    foreach((array)$pages as $page) {
        // grouping - skip first page if pages are filtered by parent id
        // and grouping is not set to page
        if($extrainfo['parent'] == $page['id'] && $extrainfo['groupby'] != 'page') {
            continue;
        }
        // skip pages which are disabled for display in menu
        if(in_array($page['parentPageId'], $blocked) || !$page['isInMenu']) {
            $blocked[] = $page['id'];
            continue;
        }
        $links[$page['id']] = array(
            $lang => array(
                'id' => $idoffset+$page['id'],
                'name' => isset($page['translatedTitle']) && !empty($page['translatedTitle']) ? $page['translatedTitle'] : $page['title'],
                'href' => pnModURL('Content', 'user', 'view', array('pid' => $page['id'])),
                'title' => isset($page['translatedTitle']) && !empty($page['translatedTitle']) ? $page['translatedTitle'] : $page['title'],
                'className' => '',
                'state' => $page['isInMenu'],
                'lang' => $lang,
                'lineno' => $page['position'],
                'parent' => isset($links[$page['parentPageId']][$lang]['id']) ? $links[$page['parentPageId']][$lang]['id'] : $parentNode
            )
        );
    }

    return $links;
}