<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2009, Pawel Preneta <jusuff@jabster.pl>
 * @link http://code.zikula.org/bianor/
 * @version $Id: modules.php 137 2009-10-04 20:01:25Z jusuff $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package menutree
 */

/**
 * Return list of user modules
 *
 * Syntax used in menutree
 * {menutree:ext:modules[:flat]}
 * Last param is optional
 *
 * @param array $args['item'] menu node to be replaced
 * @param string $args['lang'] current menu language
 * @param string $args['extrainfo'] additional params - if 'flat' then return links ungrouped
 * @return mixed array of links if successful, false otherwise
 */
function menutree_externalapi_modules($args)
{
    $item       = isset($args['item']) && !empty($args['item']) ? $args['item'] : null;
    $lang       = isset($args['lang']) && !empty($args['lang']) ? $args['lang'] : null;
    $extrainfo  = isset($args['extrainfo']) && !empty($args['extrainfo']) ? $args['extrainfo'] : null;
    // $item ang lang params are required
    if(!$item || !$lang) {
        return false;
    }

    // get id for first element, use api func to aviod id conflicts inside menu
    $idoffset = pnModAPIFunc('menutree','user','getidoffset',array('item' => $item['id']));
    $lineno = 0;

    $links = array();
    // if $extrainfo if 'flat' - don't group links
    if($extrainfo != 'flat') {
        $links['modules'] = array(
            $lang => array(
                'id' => $idoffset++,
                'name' => $item['name'],
                'href' => '',
                'title' => $item['title'],
                'className' => $item['className'],
                'state' => $item['state'],
                'lang' => $lang,
                'lineno' => $lineno++,
                'parent' => $item['parent']
            )
        );
    }
    // need to set parent node id - if links are grouped - use your_accont item id
    // otherwise parent id of replaced menu node
    $parentNode = $extrainfo != 'flat' ? $links['modules'][$lang]['id'] : $item['parent'];

    $mods = pnModGetUserMods();

    foreach($mods as $mod) {
        if (SecurityUtil::checkPermission("$mod[name]::", '::', ACCESS_OVERVIEW)) {
            $links[] = array(
                $lang => array(
                    'id' => $idoffset++,
                    'name' => $mod['displayname'],
                    'href' => pnModURL($mod['name'], 'user', 'main'),
                    'title' => $mod['description'],
                    'className' => '',
                    'state' => 1,
                    'lang' => $lang,
                    'lineno' => $lineno++,
                    'parent' => $parentNode
                )
            );
        }
    }

    return $links;
}
