<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2008, Pawel Preneta <jusuff@jabster.pl>
 * @link http://code.zikula.org/bianor/
 * @version $Id: function.admmenutree.php 130 2009-05-02 09:59:16Z jusuff $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package menutree
 */


/**
 * Smarty function to parse structured tree array to html unordered list
 *
 * This plugin is parsing list in admin mode - all menu langs are outputed.
 *
 * Available parameters:
 *   - data:        array with tree data (array, required)
 *   - id:          id for main UL element (string, optional)
 *   - class:       class for main UL element (string, optional)
 *   - nodeprefix:  prefix LI elements id (if empty - LI element wont have ids) (string, optional)
 *   - assign:      if set, the results are assigned to the corresponding variable instead of printed out
 *
 * Example
 *   <!--[menutree data=$menutree_content id='listid' class='menutree']-->
 *
 * will generate unordered list with id "listid" and class "menutree"
 *
 *
 * @author       Jusuff
 * @since        28/07/2005
 * @param        array       $params      All attributes passed to this function from the template
 * @param        object      &$smarty     Reference to the Smarty object
 * @return       string      unordered html list
 */

function smarty_function_admmenutree($params, &$smarty)
{
    $treeArray      = isset($params['data'])       ? $params['data'] : ''; 
    $treeId         = isset($params['id'])         ? $params['id'] : ''; 
    $treeClass      = isset($params['class'])      ? $params['class'] : ''; 
    $treeNodePrefix = isset($params['nodeprefix']) ? $params['nodeprefix'] : ''; 

    $html = _admHtmlList($treeArray,$treeNodePrefix,$treeId,$treeClass);

    if (isset($params['assign'])) {
        $smarty->assign($params['assign'], $html);
    } else {
        return $html;
    }

}

function _admHtmlList($tree,$treeNodePrefix,$treeId = '',$treeClass = '')
{
    $liHtml = array();
    foreach ($tree as $id => $tab) {
        $isDynamic = false;
        $links = array();
        foreach((array)$tab['item'] as $lang => $item) {
            $item = $tab['item'][$lang];
            $isDynamic = $item['dynamic'];

            $class = $item['state'] == 1 ? $item['className'] : 'unactive '.$item['className'];
            $linkClass = !empty($class) ? ' class="'.$class.'"' : '';
            $linkHref = 'href="'.pnVarPrepForDisplay($item['href']).'"';
            $linkLang = 'lang="'.$item['lang'].'"';
            $linkTitle = !empty($item['title']) ? ' title="'.$item['title'].'"' : '';

            $links[] = "<a {$linkHref} {$linkLang} {$linkTitle} {$linkClass}>{$item['name']}</a>";
        }
        $subhtml = !empty($tab['nodes']) ? _admHtmlList($tab['nodes'],$treeNodePrefix) : '';

        $liId = !empty($treeNodePrefix) ? ' id="'.$treeNodePrefix.$id.'"' : '';
        $liClass = $isDynamic ? 'class="dynamic"' : '';
        $links = implode('',$links);
        $liHtml[] = "<li {$liId} {$liClass}>{$links}{$subhtml}</li>";
    }

    $ulID = !empty($treeId) ? ' id="'.$treeId.'"' : '';
    $ulClass = !empty($treeClass) ? ' class="'.$treeClass.'"' : '';
    $liHtml = implode('',$liHtml);
    $html = "<ul {$ulID} {$ulClass}>{$liHtml}</ul>";

    return $html;
}
